@@ -37,12 +37,14 @@ enum find_best_w_mers_storage_enum {
  */
 template<
     typename Derived,
-    typename Container
+    typename Container,
+    typename Spec = default_spec
 >
 struct find_best_w_mers_storage_base {
-    typedef Container                            container;         ///< Container.
-    typedef typename container::iterator         iterator;          ///< Iterator type.
-    typedef typename container::const_iterator   const_iterator;    ///< Const iterator type.
+    typedef Container                                            container;         ///< Container.
+    typedef typename container::iterator                         iterator;          ///< Iterator type.
+    typedef typename container::const_iterator                   const_iterator;    ///< Const iterator type.
+    typedef typename steme_seqan_types< Spec >::global_pos_t    global_pos_t;      ///< Index a global position.
 
     container &         c;                ///< The storage for the best W-mers.
     size_t              max_to_store;     ///< Maximum to store.
@@ -81,11 +83,11 @@ struct find_best_w_mers_storage_base {
     /// Accept one W-mer, returns if was inserted
     inline
     bool
-    operator()( size_t W, int global_pos, bool rev_comp, double Z ) {
+    operator()( size_t W, global_pos_t global_pos, bool rev_comp, double Z ) {
         // work out where it would be inserted
         // \todo Don't need to construct the eval here, just check where
         // an evaluation with the given Z would be inserted
-        w_mer_evaluation eval( Z, global_pos, rev_comp );
+        w_mer_evaluation< Spec > eval( Z, global_pos, rev_comp );
         iterator insertion_point = derived().insertion_point( eval );
 
         // is there a better W-mer that overlaps?
@@ -106,11 +108,13 @@ struct find_best_w_mers_storage_base {
  */
 template<
     typename Derived,
-    typename Container
+    typename Container,
+    typename Spec = default_spec
 >
 struct find_best_w_mers_storage_Z_sorted
 {
     typedef typename Container::iterator         it;          ///< Iterator type.
+    typedef typename steme_seqan_types< Spec >::global_pos_t    global_pos_t;      ///< Index a global position.
 
     /// Constructor.
     find_best_w_mers_storage_Z_sorted()
@@ -119,9 +123,9 @@ struct find_best_w_mers_storage_Z_sorted
     /// for every better W-mer, check if it overlaps
     inline
     bool
-    does_better_overlap( size_t W, int global_pos, double Z, it wmer ) const {
+    does_better_overlap( size_t W, global_pos_t global_pos, double Z, it wmer ) const {
         BOOST_FOREACH(
-            const w_mer_evaluation & better,
+            const w_mer_evaluation< Spec > & better,
             static_cast< const Derived * >( this )->better_wmers( wmer )
         ) {
             if( wmer_overlap( better.global_pos, global_pos, W ) ) {
@@ -166,22 +170,29 @@ struct find_best_w_mers_storage_Z_sorted
 /**
  * Storage that holds best W-mers in a vector sorted by Z.
  */
+template< typename Spec >
 struct find_best_w_mers_sorted_vector_storage
 : find_best_w_mers_storage_base<
-    find_best_w_mers_sorted_vector_storage,
-    std::vector< w_mer_evaluation >
+    find_best_w_mers_sorted_vector_storage< Spec >,
+    std::vector< w_mer_evaluation< Spec > >,
+    Spec
 >,
 find_best_w_mers_storage_Z_sorted<
-    find_best_w_mers_sorted_vector_storage,
-    std::vector< w_mer_evaluation >
+    find_best_w_mers_sorted_vector_storage< Spec >,
+    std::vector< w_mer_evaluation< Spec > >,
+    Spec
 >
 {
     /// Base type.
     typedef find_best_w_mers_storage_base<
-        find_best_w_mers_sorted_vector_storage,
-        std::vector< w_mer_evaluation >
+        find_best_w_mers_sorted_vector_storage< Spec >,
+        std::vector< w_mer_evaluation< Spec > >,
+        Spec
     > base_t;
 
+    typedef typename base_t::container container;
+    typedef typename base_t::iterator iterator;
+
     /// Constructor.
     find_best_w_mers_sorted_vector_storage( container & c, size_t max_to_store )
     : base_t( c, max_to_store )
@@ -191,34 +202,34 @@ find_best_w_mers_storage_Z_sorted<
     inline
     void
     reserve( size_t space ) {
-        c.reserve( space );
+        base_t::c.reserve( space );
     }
 
     /// Get a bound on where to insert the evaluation.
     inline
     iterator
-    insertion_point( w_mer_evaluation eval ) {
-        return std::upper_bound( c.begin(), c.end(), eval, std::greater< w_mer_evaluation >() );
+    insertion_point( w_mer_evaluation< Spec > eval ) {
+        return std::upper_bound( base_t::c.begin(), base_t::c.end(), eval, std::greater< w_mer_evaluation< Spec > >() );
     }
 
     /// Get a range containing those W-mers that are better than the given W-mer
     inline
     boost::iterator_range< iterator >
     better_wmers( iterator wmer ) const {
-        return boost::make_iterator_range( c.begin(), wmer );
+        return boost::make_iterator_range( base_t::c.begin(), wmer );
     }
 
     /// Insert the evaluation
     /// \return Worst Z
     inline
     double
-    insert( iterator insertion_point, const w_mer_evaluation & eval ) {
-        if( num_stored == max_to_store ) {
-            c.resize( max_to_store - 1 );
-            c.insert( insertion_point, eval );
+    insert( iterator insertion_point, const w_mer_evaluation< Spec > & eval ) {
+        if( base_t::num_stored == base_t::max_to_store ) {
+            base_t::c.resize( base_t::max_to_store - 1 );
+            base_t::c.insert( insertion_point, eval );
         } else {
-            c.insert( insertion_point, eval );
-            ++num_stored;
+            base_t::c.insert( insertion_point, eval );
+            ++base_t::num_stored;
         }
         return get_worst();
     }
@@ -228,7 +239,7 @@ find_best_w_mers_storage_Z_sorted<
     inline
     iterator
     erase( iterator wmer ) {
-        c.erase( wmer );
+        base_t::c.erase( wmer );
         return wmer;
     }
 
@@ -236,7 +247,7 @@ find_best_w_mers_storage_Z_sorted<
     inline
     double
     get_worst() const {
-        return c.rbegin()->Z;
+        return base_t::c.rbegin()->Z;
     }
 };
 
@@ -244,22 +255,28 @@ find_best_w_mers_storage_Z_sorted<
 /**
  * Storage that holds best W-mers in a list sorted by Z.
  */
+template< typename Spec >
 struct find_best_w_mers_sorted_list_storage
 : find_best_w_mers_storage_base<
-    find_best_w_mers_sorted_list_storage,
-    std::list< w_mer_evaluation >
+    find_best_w_mers_sorted_list_storage< Spec >,
+    std::list< w_mer_evaluation< Spec > >,
+    Spec
 >,
 find_best_w_mers_storage_Z_sorted<
-    find_best_w_mers_sorted_list_storage,
-    std::list< w_mer_evaluation >
+    find_best_w_mers_sorted_list_storage< Spec >,
+    std::list< w_mer_evaluation< Spec > >,
+    Spec
 >
 {
     /// Base type.
     typedef find_best_w_mers_storage_base<
-        find_best_w_mers_sorted_list_storage,
-        std::list< w_mer_evaluation >
+        find_best_w_mers_sorted_list_storage< Spec >,
+        std::list< w_mer_evaluation< Spec > >
     > base_t;
 
+    typedef typename base_t::container container;
+    typedef typename base_t::iterator iterator;
+
     /// Constructor.
     find_best_w_mers_sorted_list_storage( container & c, size_t max_to_store )
     : base_t( c, max_to_store )
@@ -274,28 +291,28 @@ find_best_w_mers_storage_Z_sorted<
     /// Get a bound on where to insert the evaluation.
     inline
     iterator
-    insertion_point( w_mer_evaluation eval ) {
-        return std::upper_bound( c.begin(), c.end(), eval, std::greater< w_mer_evaluation >() );
+    insertion_point( w_mer_evaluation< Spec > eval ) {
+        return std::upper_bound( base_t::c.begin(), base_t::c.end(), eval, std::greater< w_mer_evaluation< Spec > >() );
     }
 
     /// Get a range containing those W-mers that are better than the given W-mer
     inline
     boost::iterator_range< iterator >
     better_wmers( iterator wmer ) const {
-        return boost::make_iterator_range( c.begin(), wmer );
+        return boost::make_iterator_range( base_t::c.begin(), wmer );
     }
 
     /// Insert the evaluation
     /// \return Worst Z
     inline
     double
-    insert( iterator insertion_point, const w_mer_evaluation & eval ) {
-        if( num_stored == max_to_store ) {
-            c.resize( max_to_store - 1 );
-            c.insert( insertion_point, eval );
+    insert( iterator insertion_point, const w_mer_evaluation< Spec > & eval ) {
+        if( base_t::num_stored == base_t::max_to_store ) {
+            base_t::c.resize( base_t::max_to_store - 1 );
+            base_t::c.insert( insertion_point, eval );
         } else {
-            c.insert( insertion_point, eval );
-            ++num_stored;
+            base_t::c.insert( insertion_point, eval );
+            ++base_t::num_stored;
         }
         return get_worst();
     }
@@ -305,7 +322,7 @@ find_best_w_mers_storage_Z_sorted<
     inline
     iterator
     erase( iterator wmer ) {
-        c.erase( wmer++ );
+        base_t::c.erase( wmer++ );
         return wmer;
     }
 
@@ -313,7 +330,7 @@ find_best_w_mers_storage_Z_sorted<
     inline
     double
     get_worst() const {
-        return c.rbegin()->Z;
+        return base_t::c.rbegin()->Z;
     }
 };
 
@@ -321,22 +338,29 @@ find_best_w_mers_storage_Z_sorted<
 /**
  * Storage that holds best W-mers in a set sorted by Z.
  */
+template< typename Spec >
 struct find_best_w_mers_set_storage
 : find_best_w_mers_storage_base<
-    find_best_w_mers_set_storage,
-    std::set< w_mer_evaluation, std::greater< w_mer_evaluation > >
+    find_best_w_mers_set_storage< Spec >,
+    std::set< w_mer_evaluation< Spec >, std::greater< w_mer_evaluation< Spec > > >,
+    Spec
 >,
 find_best_w_mers_storage_Z_sorted<
-    find_best_w_mers_set_storage,
-    std::set< w_mer_evaluation, std::greater< w_mer_evaluation > >
+    find_best_w_mers_set_storage< Spec >,
+    std::set< w_mer_evaluation< Spec >, std::greater< w_mer_evaluation< Spec > > >,
+    Spec
 >
 {
 	/// Base type.
 	typedef find_best_w_mers_storage_base<
-		find_best_w_mers_set_storage,
-		std::set< w_mer_evaluation, std::greater< w_mer_evaluation > >
+		find_best_w_mers_set_storage< Spec >,
+		std::set< w_mer_evaluation< Spec >, std::greater< w_mer_evaluation< Spec > > >,
+		Spec
 	> base_t;
 
+    typedef typename base_t::container container;
+    typedef typename base_t::iterator iterator;
+
 	/// Constructor.
 	find_best_w_mers_set_storage( container & c, size_t max_to_store )
 	: base_t( c, max_to_store )
@@ -351,31 +375,31 @@ find_best_w_mers_storage_Z_sorted<
 	/// Get a bound on where to insert the evaluation.
 	inline
 	iterator
-	insertion_point( w_mer_evaluation eval ) {
-		return c.upper_bound( eval );
+	insertion_point( w_mer_evaluation< Spec > eval ) {
+		return base_t::c.upper_bound( eval );
 	}
 
 	/// Get a range containing those W-mers that are better than the given W-mer
 	inline
 	boost::iterator_range< iterator >
 	better_wmers( iterator wmer ) const {
-		return boost::make_iterator_range( c.begin(), wmer );
+		return boost::make_iterator_range( base_t::c.begin(), wmer );
 	}
 
 	/// Insert the evaluation
 	/// \return Worst Z
 	inline
 	double
-	insert( iterator insertion_point, const w_mer_evaluation & eval ) {
-		if( num_stored == max_to_store ) {
-			c.insert( insertion_point, eval );
+	insert( iterator insertion_point, const w_mer_evaluation< Spec > & eval ) {
+		if( base_t::num_stored == base_t::max_to_store ) {
+		    base_t::c.insert( insertion_point, eval );
 			// remove worst
-			iterator to_erase = c.end();
+			iterator to_erase = base_t::c.end();
 			--to_erase;
-			c.erase( to_erase );
+			base_t::c.erase( to_erase );
 		} else {
-			c.insert( insertion_point, eval );
-			++num_stored;
+		    base_t::c.insert( insertion_point, eval );
+			++base_t::num_stored;
 		}
 		return get_worst();
 	}
@@ -385,7 +409,7 @@ find_best_w_mers_storage_Z_sorted<
 	inline
 	iterator
 	erase( iterator wmer ) {
-		c.erase( wmer++ );
+	    base_t::c.erase( wmer++ );
 		return wmer;
 	}
 
@@ -393,7 +417,7 @@ find_best_w_mers_storage_Z_sorted<
 	inline
 	double
 	get_worst() const {
-		return c.rbegin()->Z;
+		return base_t::c.rbegin()->Z;
 	}
 };
 
@@ -401,11 +425,13 @@ find_best_w_mers_storage_Z_sorted<
 /**
  * Storage that holds best W-mers sorted by position and score.
  */
+template< typename Spec >
 struct find_best_w_mers_store_all
 {
 public:
     /// Container type that is part of the interface.
-    typedef std::vector< w_mer_evaluation > container;
+    typedef std::vector< w_mer_evaluation< Spec > > container;
+    typedef typename steme_seqan_types< Spec >::global_pos_t    global_pos_t;      ///< Index a global position.
 
     container &            c;                ///< The container we use to store the best W-mers as part of the interface.
     int                    max_to_store;     ///< Maximum number to store.
@@ -443,8 +469,8 @@ public:
     /// Accept one W-mer, returns true iff it was inserted
     inline
     bool
-    operator()( size_t W, int global_pos, bool rev_comp, double Z ) {
-        c.push_back( w_mer_evaluation( Z, global_pos, rev_comp ) );
+    operator()( size_t W, global_pos_t global_pos, bool rev_comp, double Z ) {
+        c.push_back( w_mer_evaluation< Spec >( Z, global_pos, rev_comp ) );
         if( worst > Z ) {
             worst = Z;
         }
@@ -460,12 +486,12 @@ public:
         std::cout << "Got " << c.size() << " W-mers at end of descent.\n";
         boost::timer t;
 #endif // STEME_LOG_TIMING_INFO
-        boost::sort( c, w_mer_evaluation::comp_score() );
+        boost::sort( c, w_mer_evaluation< Spec >::comp_score() );
 #ifdef STEME_LOG_TIMING_INFO
         std::cout << "Sort took " << t.elapsed() << " seconds.\n";
         t.restart();
 #endif // STEME_LOG_TIMING_INFO
-        w_mer_evaluation::vec _c;
+        typename w_mer_evaluation< Spec >::vec _c;
         output_nonoverlapping_from_score_sorted( c.begin(), c.end(), W, new_size, std::back_inserter( _c ) );
         std::swap( c, _c );
 #ifdef STEME_LOG_TIMING_INFO
@@ -481,6 +507,7 @@ public:
 /**
  * Storage that holds best W-mers sorted by position and score.
  */
+template< typename Spec >
 struct find_best_w_mers_multi_index_storage
 {
 protected:
@@ -489,44 +516,59 @@ protected:
 
     typedef ::boost::multi_index::tag< score    > score_tag;    ///< Multi-index tag for score.
     typedef ::boost::multi_index::tag< position > position_tag; ///< Multi-index tag for position.
+    typedef typename steme_seqan_types< Spec >::global_pos_t    global_pos_t;      ///< Index a global position.
 
     /// Compares using positions
     struct comp_position
     {
         // compare a position and an eval
-        bool operator()( int global_pos, const w_mer_evaluation & eval ) const { return global_pos < eval.global_pos; }
+        bool operator()( global_pos_t global_pos, const w_mer_evaluation< Spec > & eval ) const { return global_pos < eval.global_pos; }
 
         // compare an eval and a score
-        bool operator()( const w_mer_evaluation & eval, int global_pos ) const { return eval.global_pos < global_pos; }
+        bool operator()( const w_mer_evaluation< Spec > & eval, global_pos_t global_pos ) const { return eval.global_pos < global_pos; }
     };
 
     /// Compares using scores
     struct comp_score
     {
         // compare a score and an eval
-        bool operator()( double Z, const w_mer_evaluation & eval ) const { return Z < eval.Z; }
+        bool operator()( double Z, const w_mer_evaluation< Spec > & eval ) const { return Z < eval.Z; }
 
         // compare an eval and a score
-        bool operator()( const w_mer_evaluation & eval, double Z ) const { return eval.Z < Z; }
+        bool operator()( const w_mer_evaluation< Spec > & eval, double Z ) const { return eval.Z < Z; }
     };
 
     /// Container indexed by score and by position separately.
     typedef boost::multi_index_container<
-        w_mer_evaluation,
+        w_mer_evaluation< Spec >,
         ::boost::multi_index::indexed_by<
-            ::boost::multi_index::ordered_non_unique< score_tag   , ::boost::multi_index::member< w_mer_evaluation, double, &w_mer_evaluation::Z          > >, // w-mers by score
-            ::boost::multi_index::ordered_non_unique< position_tag, ::boost::multi_index::member< w_mer_evaluation, int   , &w_mer_evaluation::global_pos > >  // w-mers by position
+            ::boost::multi_index::ordered_non_unique<
+                score_tag,
+                ::boost::multi_index::member<
+                    w_mer_evaluation< Spec >,
+                    double,
+                    &w_mer_evaluation< Spec >::Z
+                >
+            >, // w-mers by score
+            ::boost::multi_index::ordered_non_unique<
+                position_tag,
+                ::boost::multi_index::member<
+                    w_mer_evaluation< Spec >,
+                    global_pos_t,
+                    &w_mer_evaluation< Spec >::global_pos
+                >
+            >  // w-mers by position
         >
     > mi_container;
 
-    typedef mi_container::index< score    >::type   wmers_by_score;    ///< W-mers indexed by score.
-    typedef mi_container::index< position >::type   wmers_by_position; ///< W-mers indexed by position.
-    typedef wmers_by_position::iterator             score_iterator;    ///< Iterator by score.
-    typedef wmers_by_position::iterator             position_iterator; ///< Iterator by position.
+    typedef typename mi_container::template index< score    >::type   wmers_by_score;    ///< W-mers indexed by score.
+    typedef typename mi_container::template index< position >::type   wmers_by_position; ///< W-mers indexed by position.
+    typedef typename wmers_by_position::iterator                       score_iterator;    ///< Iterator by score.
+    typedef typename wmers_by_position::iterator                       position_iterator; ///< Iterator by position.
 
 public:
     /// Container type that is part of the interface.
-    typedef std::vector< w_mer_evaluation > container;
+    typedef std::vector< w_mer_evaluation< Spec > > container;
 
 protected:
     mi_container           mi;               ///< The multi-index container we use to store the best W-mers internally.
@@ -566,11 +608,11 @@ public:
     /// Accept one W-mer, returns true iff it was inserted
     inline
     bool
-    operator()( size_t W, int global_pos, bool rev_comp, double Z ) {
+    operator()( size_t W, global_pos_t global_pos, bool rev_comp, double Z ) {
         position_iterator insertion_point = sorted_by_position().upper_bound( global_pos );
 
         if( ! does_better_overlap( W, global_pos, Z, insertion_point ) ) {
-            sorted_by_position().insert( insertion_point, w_mer_evaluation( Z, global_pos, rev_comp ) );
+            sorted_by_position().insert( insertion_point, w_mer_evaluation< Spec >( Z, global_pos, rev_comp ) );
             return true;
         }
 
@@ -598,23 +640,24 @@ protected:
     inline
     wmers_by_score &
     sorted_by_score() {
-        return mi.get< score >();
+        return mi.template get< score >();
     }
 
     /// W-mers by position
     inline
     wmers_by_position &
     sorted_by_position() {
-        return mi.get< position >();
+        return mi.template get< position >();
     }
 
     /// Does a better one overlap?
     inline
     bool
-    does_better_overlap( int W, int global_pos, double Z, position_iterator insert_point ) {
+    does_better_overlap( int W, global_pos_t global_pos, double Z, position_iterator insert_point ) {
         // check those after
         position_iterator i = insert_point;
-        while( sorted_by_position().end() != i && i->global_pos - global_pos < W ) {
+        typedef typename w_mer_evaluation< Spec >::global_pos_t global_pos_t;
+        while( sorted_by_position().end() != i && i->global_pos < global_pos + global_pos_t( W ) ) {
             if( i->Z >= Z ) {
                 return true;
             }
@@ -624,7 +667,7 @@ protected:
         if( sorted_by_position().begin() != insert_point ) {
             i = insert_point;
             --i;
-            while( global_pos - i->global_pos < W ) {
+            while( global_pos < i->global_pos + global_pos_t( W ) ) {
                 if( i->Z >= Z ) {
                     return true;
                 }
@@ -643,14 +686,16 @@ protected:
 /**
  * Storage that holds best W-mers sorted by position and score.
  */
+template< typename Spec >
 struct find_best_w_mers_heap_storage
 {
 protected:
 public:
-    typedef std::vector< w_mer_evaluation > container; ///< Container type that is part of the interface.
+    typedef std::vector< w_mer_evaluation< Spec > > container; ///< Container type that is part of the interface.
+    typedef typename steme_seqan_types< Spec >::global_pos_t    global_pos_t;      ///< Index a global position.
 
 protected:
-    typedef myrrh::store_n_best< container, w_mer_evaluation::comp_score > best_n_t; ///< Stores best W-mers.
+    typedef myrrh::store_n_best< container, typename w_mer_evaluation< Spec >::comp_score > best_n_t; ///< Stores best W-mers.
 
     best_n_t               best_n;           ///< Stores best W-mers
 
@@ -685,8 +730,8 @@ public:
     /// Accept one W-mer, returns true iff it was inserted
     inline
     bool
-    operator()( size_t W, int global_pos, bool rev_comp, double Z ) {
-        return best_n( w_mer_evaluation( Z, global_pos, rev_comp ) );
+    operator()( size_t W, global_pos_t global_pos, bool rev_comp, double Z ) {
+        return best_n( w_mer_evaluation< Spec >( Z, global_pos, rev_comp ) );
     }
 
     /// Remove any overlapping W-mers and discard worst W-mers.
@@ -697,12 +742,12 @@ public:
 #ifdef STEME_LOG_TIMING_INFO
         boost::timer t;
 #endif // STEME_LOG_TIMING_INFO
-        boost::sort( best_n._storage, w_mer_evaluation::comp_score() );
+        boost::sort( best_n._storage, typename w_mer_evaluation< Spec >::comp_score() );
 #ifdef STEME_LOG_TIMING_INFO
         std::cout << "Sort took " << t.elapsed() << " seconds.\n";
         t.restart();
 #endif // STEME_LOG_TIMING_INFO
-        erase_overlapping_from_score_sorted( best_n._storage, W, new_size );
+        erase_overlapping_from_score_sorted< Spec >( best_n._storage, W, new_size );
 #ifdef STEME_LOG_TIMING_INFO
         std::cout << "Removing overlaps took " << t.elapsed() << " seconds.\n";
 #endif // STEME_LOG_TIMING_INFO
@@ -712,50 +757,50 @@ public:
 
 
 /// Specialised to choose overlap policy type.
-template< find_best_w_mers_storage_enum Storage >
+template< find_best_w_mers_storage_enum Storage, typename Spec >
 struct find_best_w_mers_storage_selector {
 };
 
 
 /// Specialisation to choose store all for find best W-mers.
-template<>
-struct find_best_w_mers_storage_selector< store_all > {
-    typedef find_best_w_mers_store_all type; ///< Type that holds best W-mers.
+template< typename Spec >
+struct find_best_w_mers_storage_selector< store_all, Spec > {
+    typedef find_best_w_mers_store_all< Spec > type; ///< Type that holds best W-mers.
 };
 
 
 /// Specialisation to choose set storage for find best W-mers.
-template<>
-struct find_best_w_mers_storage_selector< store_in_set > {
-    typedef find_best_w_mers_set_storage type; ///< Type that holds best W-mers.
+template< typename Spec >
+struct find_best_w_mers_storage_selector< store_in_set, Spec > {
+    typedef find_best_w_mers_set_storage< Spec > type; ///< Type that holds best W-mers.
 };
 
 
 /// Specialisation to choose sorted vector storage for find best W-mers.
-template<>
-struct find_best_w_mers_storage_selector< store_in_sorted_vector > {
-    typedef find_best_w_mers_sorted_vector_storage type; ///< Type that holds best W-mers.
+template< typename Spec >
+struct find_best_w_mers_storage_selector< store_in_sorted_vector, Spec > {
+    typedef find_best_w_mers_sorted_vector_storage< Spec > type; ///< Type that holds best W-mers.
 };
 
 
 /// Specialisation to choose sorted list storage for find best W-mers.
-template<>
-struct find_best_w_mers_storage_selector< store_in_sorted_list > {
-    typedef find_best_w_mers_sorted_list_storage type; ///< Type that holds best W-mers.
+template< typename Spec >
+struct find_best_w_mers_storage_selector< store_in_sorted_list, Spec > {
+    typedef find_best_w_mers_sorted_list_storage< Spec > type; ///< Type that holds best W-mers.
 };
 
 
 /// Specialisation to choose heap storage for find best W-mers.
-template<>
-struct find_best_w_mers_storage_selector< store_in_heap > {
-    typedef find_best_w_mers_heap_storage type; ///< Type that holds best W-mers.
+template< typename Spec >
+struct find_best_w_mers_storage_selector< store_in_heap, Spec > {
+    typedef find_best_w_mers_heap_storage< Spec > type; ///< Type that holds best W-mers.
 };
 
 
 /// Specialisation to choose multi-index storage for find best W-mers.
-template<>
-struct find_best_w_mers_storage_selector< store_using_multi_index > {
-    typedef find_best_w_mers_multi_index_storage type; ///< Type that holds best W-mers.
+template< typename Spec >
+struct find_best_w_mers_storage_selector< store_using_multi_index, Spec > {
+    typedef find_best_w_mers_multi_index_storage< Spec > type; ///< Type that holds best W-mers.
 };
 
 
@@ -782,20 +827,19 @@ struct find_best_w_mers_storage_selector< store_using_multi_index > {
  */
 template<
 	find_best_w_mers_storage_enum    Storage = DEFAULT_FIND_WMER_STORAGE,
-	typename                         Spec = default_spec
+	typename                        Spec = default_spec
 >
 struct find_best_w_mers
 : descender_preferred_order< find_best_w_mers< Storage, Spec >, Spec >
 {
-
-	typedef Spec                                                          spec;              ///< Specifications.
-	STEME_TYPEDEF_SEQAN_TYPES( spec )
-	typedef find_best_w_mers< Storage, Spec >                             this_t;            ///< This type.
-	typedef boost::shared_ptr< this_t >                                   ptr;               ///< Pointer.
-    typedef tree_descender< this_t, Spec >                                base_t;            ///< Base type.
-	typedef typename find_best_w_mers_storage_selector< Storage >::type   storage_policy;    ///< Storage policy type.
-	typedef typename storage_policy::container                            container_t;       ///< Container type to store best W-mers.
-	typedef boost::shared_ptr< container_t >                              container_ptr;     ///< Pointer to best W-mers.
+	STEME_TYPEDEF_SEQAN_TYPES( Spec )
+	typedef find_best_w_mers< Storage, Spec >                                    this_t;            ///< This type.
+	typedef boost::shared_ptr< this_t >                                           ptr;               ///< Pointer.
+    typedef tree_descender< this_t, Spec >                                        base_t;            ///< Base type.
+	typedef typename find_best_w_mers_storage_selector< Storage, Spec >::type   storage_policy;    ///< Storage policy type.
+	typedef typename storage_policy::container                                   container_t;       ///< Container type to store best W-mers.
+	typedef boost::shared_ptr< container_t >                                      container_ptr;     ///< Pointer to best W-mers.
+	typedef w_mer_evaluation< Spec >                                              wmer_eval_t;
 
 	size_t                               num_to_find;      ///< Number to find.
 	container_ptr                        best_w_mers;      ///< Best w_mers.
@@ -860,7 +904,7 @@ struct find_best_w_mers
 	inline
 	void
 	handle_W_mer_occurrence_one_strand(
-		int global_pos,
+	    global_pos_t global_pos,
 		double Z_strand,
 		double g_n,
 		double lambda,
